'use strict';

const path = require('path'),
  LOG_PREFIX = `"${path.basename(__filename)}":`,
  log = require('./logger'),
//verbose = log.verbose.bind(log, LOG_PREFIX),
  info = log.info.bind(log, LOG_PREFIX),
  error = log.error.bind(log, LOG_PREFIX),

  JWTGenerator = require('./jwt-generator'),
  Promise = require('bluebird'),
  retry = require('bluebird-retry'),
  getMac = Promise.promisify(require('getmac').getMac);

let _mac,
  _jwtGenerator;

const make = (loginUrl, privateKey, issuer, useRetry, generatorFunc) => {
  const promiseFunc = () => {
    info('promiseFunc called.');
    return Promise
      .resolve(_mac || getMac())
      .then(mac => {
        info(`mac: ${mac}`);
        _mac = mac;
        _jwtGenerator = _jwtGenerator || new JWTGenerator(loginUrl, privateKey, issuer || `urn:mac://${_mac}`);
        return generatorFunc(_jwtGenerator, _mac);
      })
      .catch(err => {
          error(`error in callIt.  err: ${err}`);
          throw err;
        }
      );
  };
  info(`make called.  loginUrl: ${loginUrl}, privateKey: ${!!privateKey}, issuer: ${issuer}, generatorFunc: ${!!generatorFunc}, useRetry: ${useRetry}`);
  return useRetry ? retry(promiseFunc, {max_tries: -1}) : promiseFunc();
};

module.exports = class {
  constructor(loginUrl, privateKey, useRetry, issuer) {
    info(`constructor called.  loginUrl: ${loginUrl}, privateKey: ${!!privateKey}, useRetry: ${useRetry}, issuer: ${issuer}`);
    this.loginUrl = loginUrl;
    this.privateKey = privateKey;
    this.issuer = issuer;
    this.useRetry = useRetry === true;
  }

  makeToken(subject, audience, payload, expiresIn) {
    info(`makeToken called.  subject: ${subject}, audience: ${audience}, payload: ${payload}, expiresIn: ${expiresIn}`);

    const self = this;
    return make(
      self.loginUrl, self.privateKey, self.issuer, self.useRetry,
      (jwtGenerator, mac) => jwtGenerator.make(subject, audience, payload || {mac: mac}, expiresIn));
  }

  makeNewToken(subject, audience, payload, expiresIn) {
    info(`makeNewToken called.  subject: ${subject}, audience: ${audience}, payload: ${payload}, expiresIn: ${expiresIn}`);

    const self = this;
    return make(
      self.loginUrl, self.privateKey, self.issuer, self.useRetry,
      (jwtGenerator, mac) => jwtGenerator.makeNew(subject, audience, payload || {mac: mac}, expiresIn));
  }
};
